@@ -42,6 +42,7 @@ import os
 import re
 import locale
 import pprint
+import copy
 
 from threading import Thread
 
@@ -100,6 +101,8 @@ MODEL_PATH = 'model'
 PREDICTION_PATH = 'prediction'
 EVALUATION_PATH = 'evaluation'
 ENSEMBLE_PATH = 'ensemble'
+BATCH_PREDICTION_PATH = 'batchprediction'
+
 
 # Resource Ids patterns
 ID_PATTERN = '[a-f0-9]{24}'
@@ -110,14 +113,17 @@ MODEL_RE = re.compile(r'^(public/)?%s/%s$|^shared/model/[a-zA-Z0-9]{27}$' % (
 PREDICTION_RE = re.compile(r'^%s/%s$' % (PREDICTION_PATH, ID_PATTERN))
 EVALUATION_RE = re.compile(r'^%s/%s$' % (EVALUATION_PATH, ID_PATTERN))
 ENSEMBLE_RE = re.compile(r'^%s/%s$' % (ENSEMBLE_PATH, ID_PATTERN))
+BATCH_PREDICTION_RE = re.compile(r'^%s/%s$' % (BATCH_PREDICTION_PATH,
+                                               ID_PATTERN))
 RESOURCE_RE = {
     'source': SOURCE_RE,
     'dataset': DATASET_RE,
     'model': MODEL_RE,
     'prediction': PREDICTION_RE,
     'evaluation': EVALUATION_RE,
-    'ensemble': ENSEMBLE_RE}
-
+    'ensemble': ENSEMBLE_RE,
+    'batchprediction': BATCH_PREDICTION_RE}
+DOWNLOAD_DIR = '/download'
 
 # Headers
 SEND_JSON = {'Content-Type': 'application/json;charset=utf-8'}
@@ -226,19 +232,26 @@ def get_prediction_id(prediction):
 
 
 def get_evaluation_id(evaluation):
-    """Returns a evaluation/id.
+    """Returns an evaluation/id.
 
     """
     return get_resource(EVALUATION_RE, evaluation)
 
 
 def get_ensemble_id(ensemble):
-    """Returns a ensemble/id.
+    """Returns an ensemble/id.
 
     """
     return get_resource(ENSEMBLE_RE, ensemble)
 
 
+def get_batch_prediction_id(batch_prediction):
+    """Returns a batchprediction/id.
+
+    """
+    return get_resource(BATCH_PREDICTION_RE, batch_prediction)
+
+
 def get_resource_id(resource):
     """Returns the resource id if it falls in one of the registered types
 
@@ -286,7 +299,8 @@ def get_status(resource):
     return status
 
 
-def check_resource(resource, get_method, query_string='', wait_time=1):
+def check_resource(resource, get_method, query_string='', wait_time=1,
+                   retries=None):
     """Waits until a resource is finished.
 
        Given a resource and its corresponding get_method
@@ -296,15 +310,18 @@ def check_resource(resource, get_method, query_string='', wait_time=1):
            prediction, api.get_prediction
            evaluation, api.get_evaluation
            ensemble, api.get_ensemble
+           batch_prediction, api.get_batch_prediction
        it calls the get_method on the resource with the given query_string
        and waits with sleeping intervals of wait_time
        until the resource is in a final state (either FINISHED
-       or FAULTY)
+       or FAULTY). The number of retries can be limited using the retries
+       parameter.
 
     """
     def get_kwargs(resource_id):
         if not (EVALUATION_RE.match(resource_id) or
-                PREDICTION_RE.match(resource_id)):
+                PREDICTION_RE.match(resource_id) or
+                BATCH_PREDICTION_RE.match(resource_id)):
             return {'query_string': query_string}
         return {}
 
@@ -319,7 +336,9 @@ def check_resource(resource, get_method, query_string='', wait_time=1):
             raise ValueError("Failed to extract a valid resource id to check.")
         kwargs = get_kwargs(resource_id)
 
-    while True:
+    count = 0
+    while retries is None or count < retries:
+        count += 1
         status = get_status(resource)
         code = status['code']
         if code == FINISHED:
@@ -328,6 +347,7 @@ def check_resource(resource, get_method, query_string='', wait_time=1):
             raise ValueError(status)
         time.sleep(wait_time)
         resource = get_method(resource, **kwargs)
+    return resource
 
 
 def error_message(resource, resource_type='resource', method=None):
@@ -347,6 +367,9 @@ def error_message(resource, resource_type='resource', method=None):
         if ('status' in error_info and
                 'message' in error_info['status']):
             error = error_info['status']['message']
+            extra = error_info['status'].get('extra', None)
+            if extra is not None:
+                error += ": %s" % extra
         if code == HTTP_NOT_FOUND and method == 'get':
             alternate_message = ''
             if BIGML_DOMAIN != 'bigml.io':
@@ -390,6 +413,41 @@ def assign_dir(path):
         return None
 
 
+def stream_copy(response, filename):
+    """Copies the contents of a response stream to a local file.
+
+    """
+    file_size = 0
+    path = os.path.dirname(filename)
+    check_dir(path)
+    try:
+        with open(filename, 'wb') as file_handle:
+            for chunk in response.iter_content(chunk_size=1024):
+                if chunk:
+                    file_handle.write(chunk)
+                    file_handle.flush()
+                    file_size += len(chunk)
+    except IOError:
+        file_size = 0
+    return file_size
+
+
+def http_ok(resource):
+    """Checking the validity of the http return code
+
+    """
+    if 'code' in resource:
+        return resource['code'] in [HTTP_OK, HTTP_CREATED]
+
+
+def count(listing):
+    """Count of existing resources
+
+    """
+    if 'meta' in listing and 'query_total' in listing['meta']:
+        return listing['meta']['query_total']
+
+
 ##############################################################################
 #
 # Patch for requests
@@ -488,10 +546,18 @@ class BigML(object):
         self.prediction_url = self.prediction_url + PREDICTION_PATH
         self.evaluation_url = self.url + EVALUATION_PATH
         self.ensemble_url = self.url + ENSEMBLE_PATH
+        self.batch_prediction_url = self.url + BATCH_PREDICTION_PATH
 
         if set_locale:
             locale.setlocale(locale.LC_ALL, DEFAULT_LOCALE)
         self.storage = assign_dir(storage)
+        self.getters = {
+            'source': self.get_source,
+            'dataset': self.get_dataset,
+            'model': self.get_model,
+            'ensemble': self.get_ensemble,
+            'prediction': self.get_prediction,
+            'evaluation': self.get_evaluation}
 
     def _create(self, url, body, verify=VERIFY):
         """Creates a new remote resource.
@@ -525,7 +591,6 @@ class BigML(object):
                                          headers=SEND_JSON,
                                          data=body, verify=verify)
                 code = response.status_code
-
                 if code in [HTTP_CREATED, HTTP_OK]:
                     if 'location' in response.headers:
                         location = response.headers['location']
@@ -585,7 +650,7 @@ class BigML(object):
                 else "?username=%s;api_key=%s" % (
                     shared_username, shared_api_key))
         try:
-            response = requests.get(url + self.auth + query_string,
+            response = requests.get(url + auth + query_string,
                                     headers=ACCEPT_JSON,
                                     verify=VERIFY)
             code = response.status_code
@@ -644,6 +709,7 @@ class BigML(object):
                 "code": code,
                 "message": "The resource couldn't be listed"}}
         try:
+
             response = requests.get(url + self.auth + query_string,
                                     headers=ACCEPT_JSON, verify=VERIFY)
             code = response.status_code
@@ -770,6 +836,88 @@ class BigML(object):
             'code': code,
             'error': error}
 
+    def _download(self, url, filename=None):
+        """Retrieves a remote file.
+
+        Uses HTTP GET to download a file object with a BigML `url`.
+        """
+        code = HTTP_INTERNAL_SERVER_ERROR
+        file_object = None
+
+        response = requests.get(url + self.auth,
+                                verify=VERIFY, stream=True)
+        code = response.status_code
+
+        if code == HTTP_OK:
+            if filename is None:
+                file_object = response.raw
+            else:
+                file_size = stream_copy(response, filename)
+                if file_size == 0:
+                    LOGGER.error("Error copying file to %s" % filename)
+                else:
+                    file_object = filename
+        elif code in [HTTP_BAD_REQUEST, HTTP_UNAUTHORIZED, HTTP_NOT_FOUND]:
+            error = response.content
+            LOGGER.error("Error downloading: %s" % error)
+        else:
+            LOGGER.error("Unexpected error (%s)" % code)
+            code = HTTP_INTERNAL_SERVER_ERROR
+
+        return file_object
+
+    def _set_create_from_datasets_args(self, datasets, args=None,
+                                       wait_time=3, retries=10, key=None):
+        """Builds args dictionary for the create call from a `dataset` or a
+           list of `datasets`.
+
+        """
+        dataset_ids = []
+        if not isinstance(datasets, list):
+            origin_datasets = [datasets]
+        else:
+            origin_datasets = datasets
+        for dataset in origin_datasets:
+            check_resource_type(dataset, DATASET_PATH,
+                                message="A dataset id is needed to create a"
+                                        " model.")
+            dataset = check_resource(dataset, self.get_dataset,
+                                     wait_time=wait_time, retries=retries)
+
+            dataset_ids.append(get_dataset_id(dataset))
+
+        if args is None:
+            create_args = {}
+        else:
+            create_args = copy.deepcopy(args)
+
+        if len(dataset_ids) == 1:
+            if key is None:
+                key = "dataset"
+            create_args.update({
+                key: dataset_ids[0]})
+        else:
+            if key is None:
+                key = "datasets"
+            create_args.update({
+                key: dataset_ids})
+
+        return create_args
+
+    def ok(self, resource):
+        """Waits until the resource is finished or faulty, updates it and
+           returns True on success
+
+        """
+        if http_ok(resource):
+            resource_type = get_resource_type(resource)
+            resource.update(check_resource(resource,
+                                           self.getters[resource_type]))
+            return True
+        else:
+            LOGGER.error("The resource couldn't be created: %s" %
+                         resource['error'])
+
     ##########################################################################
     #
     # Utils
@@ -1153,7 +1301,7 @@ class BigML(object):
     # https://bigml.com/developers/datasets
     #
     ##########################################################################
-    def create_dataset(self, source_or_dataset, args=None,
+    def create_dataset(self, source_or_datasets, args=None,
                        wait_time=3, retries=10):
         """Creates a remote dataset.
 
@@ -1164,36 +1312,36 @@ class BigML(object):
 
         """
         if args is None:
-            args = {}
-
-        resource_type = get_resource_type(source_or_dataset)
-        if resource_type == SOURCE_PATH:
-            source_id = get_source_id(source_or_dataset)
-            if source_id:
-                if wait_time > 0:
-                    count = 0
-                    while (not self.source_is_ready(source_id) and
-                           count < retries):
-                        time.sleep(wait_time)
-                        count += 1
-                args.update({
-                    "source": source_id})
-        elif resource_type == DATASET_PATH:
-            dataset_id = get_dataset_id(source_or_dataset)
-            if dataset_id:
-                if wait_time > 0:
-                    count = 0
-                    while (not self.dataset_is_ready(dataset_id) and
-                           count < retries):
-                        time.sleep(wait_time)
-                        count += 1
-            args.update({
-                "origin_dataset": dataset_id})
+            create_args = {}
         else:
-            raise Exception("A source or dataset id is needed to create a"
-                            " dataset. %s found." % resource_type)
+            create_args = copy.deepcopy(args)
+        if isinstance(source_or_datasets, list):
+            create_args = self._set_create_from_datasets_args(
+                source_or_datasets, args=create_args, wait_time=wait_time,
+                retries=retries, key="origin_datasets")
+        else:
+            resource_type = get_resource_type(source_or_datasets)
+            if resource_type == SOURCE_PATH:
+                source_id = get_source_id(source_or_datasets)
+                if source_id:
+                    if wait_time > 0:
+                        count = 0
+                        while (not self.source_is_ready(source_id) and
+                               count < retries):
+                            time.sleep(wait_time)
+                            count += 1
+                    create_args.update({
+                        "source": source_id})
+            elif resource_type == DATASET_PATH:
+                create_args = self._set_create_from_datasets_args(
+                    source_or_datasets, args=create_args, wait_time=wait_time,
+                    retries=retries, key="origin_dataset")
+            else:
+                raise Exception("A source, dataset or list of dataset ids"
+                                " are needed to create a"
+                                " dataset. %s found." % resource_type)
 
-        body = json.dumps(args)
+        body = json.dumps(create_args)
         return self._create(self.dataset_url, body)
 
     def get_dataset(self, dataset, query_string=''):
@@ -1255,30 +1403,15 @@ class BigML(object):
     # https://bigml.com/developers/models
     #
     ##########################################################################
-    def create_model(self, dataset, args=None, wait_time=3, retries=10):
-        """Creates a model.
+    def create_model(self, datasets, args=None, wait_time=3, retries=10):
+        """Creates a model from a `dataset` or a list o `datasets`.
 
         """
-        check_resource_type(dataset, DATASET_PATH,
-                            message="A dataset id is needed to create a"
-                                    " model.")
-        dataset_id = get_dataset_id(dataset)
+        create_args = self._set_create_from_datasets_args(
+            datasets, args=args, wait_time=wait_time, retries=retries)
 
-        if dataset_id:
-            if wait_time > 0:
-                count = 0
-                while (not self.dataset_is_ready(dataset_id) and
-                       count < retries):
-                    time.sleep(wait_time)
-                    count += 1
-
-            if args is None:
-                args = {}
-            args.update({
-                "dataset": dataset_id})
-
-            body = json.dumps(args)
-            return self._create(self.model_url, body)
+        body = json.dumps(create_args)
+        return self._create(self.model_url, body)
 
     def get_model(self, model, query_string='',
                   shared_username=None, shared_api_key=None):
@@ -1560,28 +1693,16 @@ class BigML(object):
     # https://bigml.com/developers/ensembles
     #
     ##########################################################################
-    def create_ensemble(self, dataset, args=None, wait_time=3, retries=10):
-        """Creates an ensemble.
+    def create_ensemble(self, datasets, args=None, wait_time=3, retries=10):
+        """Creates an ensemble from a dataset or a list of datasets.
 
         """
-        check_resource_type(dataset, DATASET_PATH,
-                            message="A dataset id is needed.")
-        dataset_id = get_dataset_id(dataset)
 
-        if dataset_id:
-            if wait_time > 0:
-                count = 0
-                while (not self.dataset_is_ready(dataset_id) and
-                       count < retries):
-                    time.sleep(wait_time)
-                    count += 1
+        create_args = self._set_create_from_datasets_args(
+            datasets, args=args, wait_time=wait_time, retries=retries)
 
-            if args is None:
-                args = {}
-            args.update({
-                "dataset": dataset_id})
-            body = json.dumps(args)
-            return self._create(self.ensemble_url, body)
+        body = json.dumps(create_args)
+        return self._create(self.ensemble_url, body)
 
     def get_ensemble(self, ensemble, query_string=''):
         """Retrieves an ensemble.
@@ -1635,3 +1756,118 @@ class BigML(object):
         ensemble_id = get_ensemble_id(ensemble)
         if ensemble_id:
             return self._delete("%s%s" % (self.url, ensemble_id))
+
+    ##########################################################################
+    #
+    # Batch Predictions
+    # https://bigml.com/developers/batch_predictions
+    #
+    ##########################################################################
+    def create_batch_prediction(self, model_or_ensemble, dataset,
+                                args=None, wait_time=3, retries=10):
+        """Creates a new batch prediction.
+
+        """
+
+        def args_update(check_resource_is_ready):
+            """Updates args when the resource is ready
+
+            """
+            if resource_id:
+                if wait_time > 0:
+                    count = 0
+                    while (not check_resource_is_ready(resource_id) and
+                           count < retries):
+                        time.sleep(wait_time)
+                        count += 1
+                args.update({
+                    resource_type: resource_id,
+                    "dataset": dataset_id})
+
+        if args is None:
+            args = {}
+        resource_type = get_resource_type(dataset)
+        if not DATASET_PATH == resource_type:
+            raise Exception("A dataset id is needed as second argument"
+                            " to create a batch prediction. %s found." %
+                            resource_type)
+        dataset_id = get_dataset_id(dataset)
+        if dataset_id:
+            if wait_time > 0:
+                count = 0
+                while (not self.dataset_is_ready(dataset_id) and
+                       count < retries):
+                    time.sleep(wait_time)
+                    count += 1
+
+        resource_type = get_resource_type(model_or_ensemble)
+        if resource_type == MODEL_PATH:
+            resource_id = get_model_id(model_or_ensemble)
+            args_update(self.model_is_ready)
+        elif resource_type == ENSEMBLE_PATH:
+            resource_id = get_ensemble_id(model_or_ensemble)
+            args_update(self.ensemble_is_ready)
+        else:
+            raise Exception("A model or ensemble id is needed as first"
+                            " argument to create a"
+                            " batch prediction. %s found." % resource_type)
+
+        body = json.dumps(args)
+        return self._create(self.batch_prediction_url, body)
+
+    def get_batch_prediction(self, batch_prediction):
+        """Retrieves a batch prediction.
+
+           The batch_prediction parameter should be a string containing the
+           batch_prediction id or the dict returned by create_batch_prediction.
+           As batch_prediction is an evolving object that is processed
+           until it reaches the FINISHED or FAULTY state, the function will
+           return a dict that encloses the batch_prediction values and state
+           info available at the time it is called.
+        """
+        check_resource_type(batch_prediction, BATCH_PREDICTION_PATH,
+                            message="A batch prediction id is needed.")
+        batch_prediction_id = get_batch_prediction_id(batch_prediction)
+        if batch_prediction_id:
+            return self._get("%s%s" % (self.url, batch_prediction_id))
+
+    def download_batch_prediction(self, batch_prediction, filename=None):
+        """Retrieves the batch predictions file.
+
+           Downloads predictions, that are stored in a remote CSV file. If
+           a path is given in filename, the contents of the file are downloaded
+           and saved locally. A file-like object is returned otherwise.
+        """
+        check_resource_type(batch_prediction, BATCH_PREDICTION_PATH,
+                            message="A batch prediction id is needed.")
+        batch_prediction_id = get_batch_prediction_id(batch_prediction)
+        if batch_prediction_id:
+            return self._download("%s%s%s" % (self.url, batch_prediction_id,
+                                              DOWNLOAD_DIR), filename=filename)
+
+    def list_batch_predictions(self, query_string=''):
+        """Lists all your batch predictions.
+
+        """
+        return self._list(self.batch_prediction_url, query_string)
+
+    def update_batch_prediction(self, batch_prediction, changes):
+        """Updates a batch prediction.
+
+        """
+        check_resource_type(batch_prediction, BATCH_PREDICTION_PATH,
+                            message="A batch prediction id is needed.")
+        batch_prediction_id = get_batch_prediction_id(batch_prediction)
+        if batch_prediction_id:
+            body = json.dumps(changes)
+            return self._update("%s%s" % (self.url, batch_prediction_id), body)
+
+    def delete_batch_prediction(self, batch_prediction):
+        """Deletes a batch prediction.
+
+        """
+        check_resource_type(batch_prediction, BATCH_PREDICTION_PATH,
+                            message="A batch prediction id is needed.")
+        batch_prediction_id = get_batch_prediction_id(batch_prediction)
+        if batch_prediction_id:
+            return self._delete("%s%s" % (self.url, batch_prediction_id))
