@@ -40,19 +40,27 @@ def setup_resources(feature):
     assert ensembles['code'] == HTTP_OK
     world.init_ensembles_count = ensembles['meta']['total_count']
 
+    batch_predictions = world.api.list_batch_predictions()
+    assert batch_predictions['code'] == HTTP_OK
+    world.init_batch_predictions_count = batch_predictions['meta']['total_count']
+
     world.sources = []
     world.datasets = []
     world.models = []
     world.predictions = []
     world.evaluations = []
     world.ensembles = []
+    world.batch_predictions = []
+
+    world.dataset_ids = []
 
 @after.each_feature
 def cleanup_resources(feature):
-    
+
     if os.path.exists('./tmp'):
         shutil.rmtree('./tmp')
 
+
     for id in world.sources:
         world.api.delete_source(id)
     world.sources = []
@@ -77,6 +85,10 @@ def cleanup_resources(feature):
         world.api.delete_ensemble(id)
     world.ensembles = []
 
+    for id in world.batch_predictions:
+        world.api.delete_batch_prediction(id)
+    world.batch_predictions = []
+
     sources = world.api.list_sources()
     assert sources['code'] == HTTP_OK
     world.final_sources_count = sources['meta']['total_count']
@@ -101,9 +113,18 @@ def cleanup_resources(feature):
     assert ensembles['code'] == HTTP_OK
     world.final_ensembles_count = ensembles['meta']['total_count']
 
+    batch_predictions = world.api.list_batch_predictions()
+    assert batch_predictions['code'] == HTTP_OK
+    world.final_batch_predictions_count = batch_predictions['meta']['total_count']
+
     assert world.final_sources_count == world.init_sources_count
     assert world.final_datasets_count == world.init_datasets_count
     assert world.final_models_count == world.init_models_count
     assert world.final_predictions_count == world.init_predictions_count
     assert world.final_evaluations_count == world.init_evaluations_count
     assert world.final_ensembles_count == world.init_ensembles_count
+    assert world.final_batch_predictions_count == world.init_batch_predictions_count
+
+@after.each_scenario
+def cleanup_resources(scenario):
+    world.dataset_ids = []
